==> v1.0.3.tar.gz: activiti-api-doclet/src/main/java/com/inteligr8/activiti/doclet/ActivitiDocFilter.java <==
/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.doclet;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import jdk.javadoc.doclet.DocletEnvironment;

/**
 * @author brian@inteligr8.com
 */
class ActivitiDocFilter {
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	private final List<ActivitiApiBeanDoc> beandocs = new LinkedList<>();
	private final DocletEnvironment docenv;
	private final boolean skipObjectMethods = true;
	
	public ActivitiDocFilter(DocletEnvironment docenv) {
		this.docenv = docenv;
	}
	
	public boolean filter(Element element) {
		if (element.getKind() != ElementKind.CLASS) {
			this.logger.trace("Skipping non-class element: {}", element);
			return false;
		}
		
		TypeElement classElement = (TypeElement)element;

		if (!classElement.getModifiers().contains(Modifier.PUBLIC)) {
			this.logger.trace("Skipping non-public class: {}", classElement);
			return false;
		}

		if (classElement.getModifiers().contains(Modifier.ABSTRACT)) {
			this.logger.trace("Skipping abstract class: {}", classElement);
			return false;
		}
		
		return this.filterPublicConcreteClass(classElement);
	}
	
	private boolean filterPublicConcreteClass(TypeElement classElement) {
		Component classComponent = classElement.getAnnotation(Component.class);
		if (classComponent == null) {
			// if we want to support XML defined beans, here is where that XML consultation would happen ...
			this.logger.trace("Skipping non-Spring bean class: {}", classElement);
			return false;
		}
		
		if (classComponent.value() == null || classComponent.value().trim().length() == 0) {
			this.logger.debug("Skipping unnamed Spring bean class: {}", classElement);
			return false;
		}

		this.addNamedBean(classComponent.value().trim(), classElement);
		return true;
	}
	
	private void addNamedBean(String beanId, TypeElement classElement) {
		this.logger.debug("Adding named bean: {} => {}", beanId, classElement);
		
		ActivitiApiBeanDoc beandoc = new ActivitiApiBeanDoc(beanId, classElement);

		// getAllTypeElements() will get inherited interfaces
		for (TypeMirror interfaceType : classElement.getInterfaces()) {
			this.logger.trace("Found interface '{}' on bean '{}'", interfaceType, beanId);
			
			switch (interfaceType.toString()) {
				case "org.activiti.engine.delegate.JavaDelegate":
					this.logger.debug("The bean '{}' is a JavaDelegate", beanId);
					beandoc.setDelegate(true);
					break;
				case "org.activiti.engine.delegate.ExecutionListener":
					this.logger.debug("The bean '{}' is a ExecutionListener", beanId);
					beandoc.setExecutionListener(true);
					break;
				case "org.activiti.engine.delegate.TaskListener":
					this.logger.debug("The bean '{}' is a TaskListener", beanId);
					beandoc.setTaskListener(true);
					break;
			}
		}
		
		// getAllMembers() will get inherited methods
		for (Element memberElement : this.docenv.getElementUtils().getAllMembers(classElement)) {
			switch (memberElement.getKind()) {
				case METHOD:
					ExecutableElement methodElement = (ExecutableElement)memberElement;
					
					if (!methodElement.getModifiers().contains(Modifier.PUBLIC)) {
						this.logger.trace("Skipping non-public method: {}#{}", classElement, methodElement.getSimpleName());
						continue;
					}
					
					if (methodElement.getModifiers().contains(Modifier.ABSTRACT)) {
						this.logger.trace("Skipping abstract method: {}#{}", classElement, methodElement.getSimpleName());
						continue;
					}
					
					if (this.skipObjectMethods && methodElement.getEnclosingElement().toString().equals("java.lang.Object")) {
						this.logger.trace("Skipping java.lang.Object methods: {}#{}", classElement, methodElement.getSimpleName());
						continue;
					}

					this.logger.debug("The bean '{}' has method '{}'", beanId, methodElement);
					beandoc.addMethod(methodElement);
					break;
				default:
			}
		}
		
		this.beandocs.add(beandoc);
	}
	
	public List<ActivitiApiBeanDoc> build() {
		// sort alphabetically
		List<ActivitiApiBeanDoc> beandocs = new LinkedList<>(this.beandocs);
		Collections.sort(beandocs, new Comparator<ActivitiApiBeanDoc>() {
			@Override
			public int compare(ActivitiApiBeanDoc beandoc1, ActivitiApiBeanDoc beandoc2) {
				// TODO sort number parts of classes inteligently: Class2 before Class10
				return beandoc1.getBeanId().compareToIgnoreCase(beandoc2.getBeanId());
			}
		});
		
		return beandocs;
	}

}

==> v1.0.4.tar.gz: activiti-api-doclet/src/main/java/com/inteligr8/activiti/doclet/ActivitiDocFilter.java <==
/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.doclet;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import jdk.javadoc.doclet.DocletEnvironment;

/**
 * @author brian@inteligr8.com
 */
class ActivitiDocFilter {
	
	private final static String INTERFACE_JAVA_DELEGATE = "org.activiti.engine.delegate.JavaDelegate";
	private final static String INTERFACE_EXECUTION_LISTENER = "org.activiti.engine.delegate.ExecutionListener";
	private final static String INTERFACE_TASK_LISTENER = "org.activiti.engine.delegate.TaskListener";
	
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	private final List<ActivitiApiBeanDoc> beandocs = new LinkedList<>();
	private final DocletEnvironment docenv;
	private final boolean skipObjectMethods = true;
	
	public ActivitiDocFilter(DocletEnvironment docenv) {
		this.docenv = docenv;
	}
	
	public boolean filter(Element element) {
		if (element.getKind() != ElementKind.CLASS) {
			this.logger.trace("Skipping non-class element: {}", element);
			return false;
		}
		
		TypeElement classElement = (TypeElement)element;

		if (!classElement.getModifiers().contains(Modifier.PUBLIC)) {
			this.logger.trace("Skipping non-public class: {}", classElement);
			return false;
		}

		if (classElement.getModifiers().contains(Modifier.ABSTRACT)) {
			this.logger.trace("Skipping abstract class: {}", classElement);
			return false;
		}
		
		return this.filterPublicConcreteClass(classElement);
	}
	
	private boolean filterPublicConcreteClass(TypeElement classElement) {
		Component classComponent = classElement.getAnnotation(Component.class);
		if (classComponent == null) {
			// if we want to support XML defined beans, here is where that XML consultation would happen ...
			this.logger.trace("Skipping non-Spring bean class: {}", classElement);
			return false;
		}
		
		if (classComponent.value() == null || classComponent.value().trim().length() == 0) {
			this.logger.debug("Skipping unnamed Spring bean class: {}", classElement);
			return false;
		}

		this.addNamedBean(classComponent.value().trim(), classElement);
		return true;
	}
	
	private void addNamedBean(String beanId, TypeElement classElement) {
		this.logger.debug("Adding named bean: {} => {}", beanId, classElement);
		
		ActivitiApiBeanDoc beandoc = new ActivitiApiBeanDoc(beanId, classElement);
		
		Set<String> delegateMethodElements = new HashSet<>(5);
		Set<String> executionListenerMethodElements = new HashSet<>(5);
		Set<String> taskListenerMethodElements = new HashSet<>(5);

		// getAllTypeElements() will get inherited interfaces
		for (TypeMirror interfaceType : classElement.getInterfaces()) {
			this.logger.trace("Found interface '{}' on bean '{}'", interfaceType, beanId);
			
			switch (interfaceType.toString()) {
				case INTERFACE_JAVA_DELEGATE:
					this.logger.debug("The bean '{}' is a JavaDelegate", beanId);
					delegateMethodElements.addAll(this.toStrings(this.docenv.getTypeUtils().asElement(interfaceType).getEnclosedElements()));
					break;
				case INTERFACE_EXECUTION_LISTENER:
					this.logger.debug("The bean '{}' is a ExecutionListener", beanId);
					executionListenerMethodElements.addAll(this.toStrings(this.docenv.getTypeUtils().asElement(interfaceType).getEnclosedElements()));
					break;
				case INTERFACE_TASK_LISTENER:
					this.logger.debug("The bean '{}' is a TaskListener", beanId);
					taskListenerMethodElements.addAll(this.toStrings(this.docenv.getTypeUtils().asElement(interfaceType).getEnclosedElements()));
					break;
			}
		}
		
		this.logger.info("delegates: {}", delegateMethodElements);
		
		// getAllMembers() will get inherited methods
		for (Element memberElement : this.docenv.getElementUtils().getAllMembers(classElement)) {
			switch (memberElement.getKind()) {
				case METHOD:
					ExecutableElement methodElement = (ExecutableElement)memberElement;
					
					if (!methodElement.getModifiers().contains(Modifier.PUBLIC)) {
						this.logger.trace("Skipping non-public method: {}.{}", classElement, methodElement.getSimpleName());
						continue;
					}
					
					if (methodElement.getModifiers().contains(Modifier.ABSTRACT)) {
						this.logger.trace("Skipping abstract method: {}.{}", classElement, methodElement.getSimpleName());
						continue;
					}
					
					if (this.skipObjectMethods && methodElement.getEnclosingElement().toString().equals("java.lang.Object")) {
						this.logger.trace("Skipping java.lang.Object method: {}.{}", classElement, methodElement.getSimpleName());
						continue;
					}
					
					if (delegateMethodElements.contains(methodElement.toString())) {
						this.logger.debug("The bean '{}' has delegate method '{}'", beanId, methodElement);
						beandoc.setDelegateMethodElement(methodElement);
					} else if (executionListenerMethodElements.contains(methodElement.toString())) {
						this.logger.debug("The bean '{}' has execution listener method '{}'", beanId, methodElement);
						beandoc.setExecutionListenerMethodElement(methodElement);
					} else if (taskListenerMethodElements.contains(methodElement.toString())) {
						this.logger.debug("The bean '{}' has task listener method '{}'", beanId, methodElement);
						beandoc.setTaskListenerMethodElement(methodElement);
					} else {
						this.logger.debug("The bean '{}' has method '{}'", beanId, methodElement);
						beandoc.addMethodElement(methodElement);
					}
					break;
				default:
			}
		}
		
		this.beandocs.add(beandoc);
	}
	
	public List<ActivitiApiBeanDoc> build() {
		// sort alphabetically
		List<ActivitiApiBeanDoc> beandocs = new LinkedList<>(this.beandocs);
		Collections.sort(beandocs, new Comparator<ActivitiApiBeanDoc>() {
			@Override
			public int compare(ActivitiApiBeanDoc beandoc1, ActivitiApiBeanDoc beandoc2) {
				// TODO sort number parts of classes inteligently: Class2 before Class10
				return beandoc1.getBeanId().compareToIgnoreCase(beandoc2.getBeanId());
			}
		});
		
		return beandocs;
	}
	
	public List<String> toStrings(Collection<? extends Element> elements) {
		if (elements == null)
			return Collections.emptyList();
		
		List<String> strs = new ArrayList<String>(elements.size());
		for (Element element : elements)
			strs.add(element.toString());
		return strs;
	}

}
